#ifndef COMPOSED_OBJECT_
#define COMPOSED_OBJECT_

#include <stdio.h>
#include <windows.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glut.h>

#include "RawObject.h"

/**
 * \brief An object that is composed of several other objects.
 *
 * This is an object which may consist of several parts. These parts can be added to the object
 * and are drawn in the same order as they were added to this object.
 */
class ComposedObject : public RawObject
{
public:
	/**
	 * Constructor: Bring the object into a consistent state.
	 */ 
	ComposedObject ();

	/**
	 * Destructor.
	 */
	~ComposedObject ();

	/** 
	 * Adds a object.
	 *
	 * \param object			The object which shall be part of this composed object.
	 */ 
	void AddObject (RawObject * object);

	/**
	 * Implementation of the Draw routine. Draws all the objects that were added to this object.
	 */
	void Draw ();

private:

	/**
	 * Structure used to store the parts of this object.
	 */
	typedef struct ComponentElem
	{
		ComponentElem * next;		/**< Pointer to the next component */
		RawObject * object;			/**< Pointer to a part of this object */
	}Components;

	/** List containing all the parts of this object */
	Components * first_component;
};

#endif