#ifndef LANDPATCH_
#define LANDPATCH_

#include <stdio.h>
#include <windows.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glut.h>

#include "VertexArrayObject.h"

/**
 * \brief Small patch of a landscape.
 *
 * A patch of a landscape which gets build from a height field. Uses vertex arrays to display itself.
 */
class LandPatch: public VertexArrayObject
{
public:
	/**
	 * Constructor: Builds the geometry for the patch. The parameters of this routines are not
	 * validated, which means that the construction will fail if they do not make sense.
	 *
	 * \param height_field			Pointer to the actual height data used to build the patch. There must
	 *								be at least 'total_x' * 'total_y' elements.
	 * \param size_x				Size of the patch along the x dimension.
	 * \param size_y				Size of the patch along the y dimension.
	 * \param offset_x				X offset into the height field (used when building a patch that is smaller
	 *								than the actual height field).
	 * \param offset_y				Y offset into the height field (used when building a patch that is smaller
	 *								than the actual height field).
	 * \param total_x				Size of the height field along the x dimension.
	 * \param total_y				Size of the height field along the y dimension.
	 * \param stride				Size of the grid squares. The x and y coordinates of the constructed
	 *								geometry range from (0,0) to ('stride'*'size_x', 'stride'*'size_y').
	 * \param xy_jump				Used to sample the height field at bigger intervals. Only every
	 *								'xy_jump'th value of the height field is taken.
	 */
	LandPatch (float * height_field, int size_x, int size_y, int offset_x, int offset_y, int total_x, int total_y, float stride, int xy_jump);

	/**
	 * Destructor.
	 */
	~LandPatch ();

private:
	LandPatch ();

	float * data;
	GLuint * indices;


};

#endif