#include "TerminalObject.h"

TerminalObject::TerminalObject (RawObject * raw_obj): GraphicalObject (raw_obj)
{
	// nothing to do
}

TerminalObject::~TerminalObject ()
{
	// do nothing
}

void TerminalObject::Draw (int time, Camera * camera, Camera * monitor, bool show_bounding_volume, int culling_mode, bool level_of_detail)
{
	Position pos;
	bool rotation;
	int culling_result;

	// return if position is not defined for the current time
	if (!(position->IsDefined (time))) return;

	// retrieve actual position
	pos = position->GetPosition (time);

	// check if we have to rotate
	rotation = position->HasStaticRotation ();


	// translate and rotate according to the actual position
	ApplyTransformation (pos, rotation);

	// cull the object against the viewing frustum
	culling_result = CullObject (camera, monitor, culling_mode, show_bounding_volume);

	// draw the object if it does not lie completely outside
	if (culling_result != C_OUTSIDE)
	{raw_object->Draw ();}

	// undo rotation and translation
	UndoTransformation (pos, rotation);
}

void TerminalObject::ComputeBoundingRadius ()
{
	bounding_radius = raw_object->GetBoundingRadius ();
}

void TerminalObject::ComputeBoundingBox ()
{
	bounding_box =  raw_object->GetBoundingBox ();
}

void TerminalObject::PrintToConsole ()
{
	Position pos = position->GetPosition (0);
	printf ("TerminalObject: {%g, %g, %g, %g, %g, %g}\n", pos.x, pos.y, pos.z, pos.roll, pos.pitch, pos.yaw);
}