#include "Utility.h"

void Utility::DrawQuad (const Point& p1, const Point& p2, const Point& p3, const Point& p4)
{
	Point midpoint;

	// calculate midpoint
	midpoint.x = (p1.x + p2.x + p3.x + p4.x) / 4;
	midpoint.y = (p1.y + p2.y + p3.y + p4.y) / 4;
	midpoint.z = (p1.z + p2.z + p3.z + p4.z) / 4;

	// do not fill the quad
	glPolygonMode (GL_FRONT_AND_BACK, GL_LINE);

	glColor4fv (color);

	glBegin (GL_QUADS);

		glVertex3f (p1.x, p1.y, p1.z);
		glVertex3f (p2.x, p2.y, p2.z);
		glVertex3f (p3.x, p3.y, p3.z);
		glVertex3f (p4.x, p4.y, p4.z);

	glEnd ();

	// reset fill mode
	glPolygonMode (GL_FRONT_AND_BACK, GL_FILL);
}

void Utility::DrawBoundingBox (BoundingBox bounding_box)
{
	// do not fill the quads
	glPolygonMode (GL_FRONT_AND_BACK, GL_LINE);

	glColor4fv (color);

	glBegin (GL_QUADS);

		// top
		glVertex3f (bounding_box.xmax, bounding_box.ymax, bounding_box.zmax);
		glVertex3f (bounding_box.xmax, bounding_box.ymin, bounding_box.zmax);
		glVertex3f (bounding_box.xmin, bounding_box.ymin, bounding_box.zmax);
		glVertex3f (bounding_box.xmin, bounding_box.ymax, bounding_box.zmax);

		// bottom
		glVertex3f (bounding_box.xmax, bounding_box.ymax, bounding_box.zmin);
		glVertex3f (bounding_box.xmax, bounding_box.ymin, bounding_box.zmin);
		glVertex3f (bounding_box.xmin, bounding_box.ymin, bounding_box.zmin);
		glVertex3f (bounding_box.xmin, bounding_box.ymax, bounding_box.zmin);

		// left
		glVertex3f (bounding_box.xmax, bounding_box.ymin, bounding_box.zmin);
		glVertex3f (bounding_box.xmax, bounding_box.ymin, bounding_box.zmax);
		glVertex3f (bounding_box.xmin, bounding_box.ymin, bounding_box.zmax);
		glVertex3f (bounding_box.xmin, bounding_box.ymin, bounding_box.zmin);

		// right
		glVertex3f (bounding_box.xmax, bounding_box.ymax, bounding_box.zmin);
		glVertex3f (bounding_box.xmax, bounding_box.ymax, bounding_box.zmax);
		glVertex3f (bounding_box.xmin, bounding_box.ymax, bounding_box.zmax);
		glVertex3f (bounding_box.xmin, bounding_box.ymax, bounding_box.zmin);

		// front
		glVertex3f (bounding_box.xmax, bounding_box.ymax, bounding_box.zmin);
		glVertex3f (bounding_box.xmax, bounding_box.ymax, bounding_box.zmax);
		glVertex3f (bounding_box.xmax, bounding_box.ymin, bounding_box.zmax);
		glVertex3f (bounding_box.xmax, bounding_box.ymin, bounding_box.zmin);

		// back
		glVertex3f (bounding_box.xmin, bounding_box.ymax, bounding_box.zmin);
		glVertex3f (bounding_box.xmin, bounding_box.ymax, bounding_box.zmax);
		glVertex3f (bounding_box.xmin, bounding_box.ymin, bounding_box.zmax);
		glVertex3f (bounding_box.xmin, bounding_box.ymin, bounding_box.zmin);

	glEnd ();

	// reset fill mode
	glPolygonMode (GL_FRONT_AND_BACK, GL_FILL);
}

void Utility::SetColor (float red, float green, float blue, float alpha)
{
	color[0] = red;
	color[1] = green;
	color[2] = blue;
	color[3] = alpha;
}

float Utility::color[4] = {1.0f, 1.0f, 1.0f, 0.0f};

Point Utility::GetPointFromBox (const BoundingBox & box, int index)
{
	Point result;
	switch (index)
	{
		case 0:		result.x = box.xmax;
					result.y = box.ymax;
					result.z = box.zmax;
					break;
		case 1:		result.x = box.xmax;
					result.y = box.ymax;
					result.z = box.zmin;
					break;
		case 2:		result.x = box.xmax;
					result.y = box.ymin;
					result.z = box.zmax;
					break;
		case 3:		result.x = box.xmax;
					result.y = box.ymin;
					result.z = box.zmin;
					break;
		case 4:		result.x = box.xmin;
					result.y = box.ymax;
					result.z = box.zmax;
					break;
		case 5:		result.x = box.xmin;
					result.y = box.ymax;
					result.z = box.zmin;
					break;
		case 6:		result.x = box.xmin;
					result.y = box.ymin;
					result.z = box.zmax;
					break;
		case 7:		result.x = box.xmin;
					result.y = box.ymin;
					result.z = box.zmin;
					break;
		default:	result.x = 0.0f;
					result.y = 0.0f;
					result.z = 0.0f;
	}

	return result;
}